#!/usr/bin/perl -w

use Fcntl ':flock';
use Cwd qw(abs_path);

my $port = shift;
if ($port) {
	if ($port !~ m/^\d+$/) {
		die "    Error: Invalid port '$port' specified!\n";
	}
}
else {
	$port = 8889;
}

my $sudo_cmd = shift;
if (!$sudo_cmd) {
	if (`lsb_release -ds 2>/dev/null` =~ m/ubuntu/i) {
		$sudo_cmd = 'sudo su -c';
	}
	else {
		$sudo_cmd = 'su -c';
	}
}

my $system_run;
my $atk_dir = abs_path($0);
$atk_dir =~ s!/[^/]+/[^/]+$!!;
my $app_data;
if ($atk_dir eq '/opt/lsb/app-testkit/manager') {
	$system_run = 1;
	$app_data = '/var/opt/lsb/app-testkit/manager';
}
else {
	$system_run = 0;
	$app_data = $atk_dir;
}

print "    The port '$port' will be used by the LSB ATK Manager web-UI server.\n";
if ($system_run and ($< != 0)) {
	print "    The command '$sudo_cmd' will be used to gain root access.\n";
	print "    If you want to change this, run $0 <port> <sudo-command>\n\n";
}
else {
	print "    If you want to change this, run $0 <port>\n\n";
}

my $LOCK_FILE = "/tmp/atk-server.lock.$port";
if (-f $LOCK_FILE) {
	if (open(LOCKFILE, $LOCK_FILE)) {
		if (!flock(LOCKFILE, LOCK_EX | LOCK_NB)) {
			my $pid = <LOCKFILE>;
			$pid =~ s/^[LG]:(\d+)$/$1/;
			my $user = `ps -p $pid -o user=`;
			$user =~ s/[\r\n]//g;
			die "    Error: An ATK Manager instance is running on the port '$port' already!\n    (Started by user '$user'; PID: $pid.)\n";
		}
		else {
			flock(LOCKFILE, LOCK_UN);
		}
		close(LOCKFILE);
	}
}

my $cmd = "mkdir \"$app_data/results\" >/dev/null 2>&1; mkdir \"$app_data/log\" >/dev/null 2>&1; $atk_dir/webui/atk-server.pl $port &";
if ($system_run and ($< != 0)) {
	$cmd = "$sudo_cmd '$cmd'";
}
my $res = system($cmd);
sleep(1);
if ($res != 0) {
	die "    Failed to start server.\n";
}
print "    Server started. Log file location:\n    $app_data/log/atk-server.log.$port\n";

# Try to open start page in a browser
my $machine = `uname -n`;
$machine =~ s/[\r\n]//g;
print "\n    The start page should be opened in a browser shortly.\n    If it doesn't open, you can load it at http://$machine:$port\n";

my $browser_not_found = 5;

# First, try the Portland's xdg-open util.
if (system('which xdg-open >/dev/null 2>&1') == 0) {
	system("xdg-open \"http://127.0.0.1:$port\" >/tmp/atk-xdg-err.$$ 2>&1");
	#An exit code of 0 indicates success.
	#1 Error in command line syntax. 
	#2 One of the files passed on the command line did not exist. 
	#3 A required tool could not be found. 
	#4 The action failed.
	$browser_not_found = $?;
	if ($browser_not_found) {
		system("cat /tmp/atk-xdg-err.$$");
	}
	unlink("/tmp/atk-xdg-err.$$");
}
else {
	# Try to find a browser by it's name.
	foreach my $br_name qw(opera /usr/bin/mozilla-firefox firefox konqueror epiphany galeon mozilla) {
		if (system("which $br_name >/dev/null 2>&1") == 0) {
			system("$br_name \"http://127.0.0.1:$port\" >/dev/null 2>&1 &");
			$browser_not_found = 0;
			last;
		}
	}
}

if ($browser_not_found) {
	print "\n    Could not find a web-browser.\n    Go to 'http://$machine:$port' to configure and launch tests.\n";
}
