#!/usr/bin/perl -w

use Fcntl ":flock";
use Cwd qw(cwd abs_path);

sub confirm($) {
	print $_[0];
	$reply = <STDIN>;
	if (($reply !~ m/^y$/i) and ($reply !~ m/^yes$/i)) {
		exit(2);
	}
}

my $port = shift;
my $port_specified;
if ($port) {
	if ($port !~ m/^\d+$/) {
		die "    Error: Invalid port '$port' specified!\n";
	}
	$port_specified = 1;
}
else {
	$port_specified = 0;
}

my $sudo_cmd = shift;
my $use_sudo = 0;
if (!$sudo_cmd) {
	if (`lsb_release -ds 2>/dev/null` =~ m/ubuntu/i) {
		$sudo_cmd = "sudo su -c";
	}
	else {
		$sudo_cmd = "su -c";
	}
}
else {
	$use_sudo = 1;
}

my $atk_dir = abs_path($0);
$atk_dir =~ s!/[^/]+/[^/]+$!!;
my $whoami = `id -un`;
$whoami =~ s/[\r\n]//g;

opendir(DIR, "/tmp") or die "    Error: Could not access /tmp!\n";
my @lock_list = readdir(DIR);
closedir(DIR);
my %all_servers_usr = (); # port => user
my %all_servers = ();     # port => pid
my %loc_servers = ();     # The same, but only for servers started by the current user
my %sys_servers = ();     # The same, but only for system-level servers (from /opt/lsb/app-testkit/manager)
foreach my $lock_file (sort(grep(m/^atk-server\.lock\.\d+$/, @lock_list))) {
	if (open(LOCKFILE, "/tmp/$lock_file")) {
		if (!flock(LOCKFILE, LOCK_EX | LOCK_NB)) {
			my $pid = <LOCKFILE>;
			$pid =~ s/^([LG]):(\d+)$/$2/;
			my $is_system = ($1 eq "G");
			$lock_file =~ m/^atk-server\.lock\.(\d+)/;
			my $srv_port = $1;
			my $user = `ps -p $pid -o user=`;
			$user =~ s/[\r\n]//g;
			$all_servers_usr{$srv_port} = $user;
			$all_servers{$srv_port} = $pid;
			$sys_servers{$srv_port} = $pid if ($is_system);
			$loc_servers{$srv_port} = $pid if (!$is_system and (($< == 0) or ($user eq $whoami)));
		}
		close(LOCKFILE);
	}
}

my $system_run = 0;

if (!$port_specified) {
	my $num_loc = scalar(keys(%loc_servers));
	my $num_sys = scalar(keys(%sys_servers));
	my $num = $num_loc + $num_sys;
	if ($num == 0) {
		if (scalar(keys(%all_servers)) > 0) {
			die "    Error: There are no running ATK servers started by you.\n";
		}
		elsif (system("ps -A | grep 'atk-server\\.pl' >/dev/null") == 0) {
			confirm("    It seems that a previous version of ATK server is running. killall will be used to stop it.\n    Continue? ");
			system("$sudo_cmd 'killall atk-server.pl'");
			exit;
		}
		else {
			die "    Error: There are no running ATK servers.\n";
		}
	}
	elsif ($num == 1) {
		if ($num_loc == 1) {
			$port = (keys(%loc_servers))[0];
		}
		else {
			$port = (keys(%sys_servers))[0];
			$system_run = 1;
		}
		confirm("    Are you sure you want to stop the server? ");
	}
	else {
		print "    There are several ATK servers running:\n";
		my @ports_loc = sort(keys(%loc_servers));
		my @ports_sys = sort(keys(%sys_servers));
		my $glob_i = 0;
		my $i = 0;
		while ($i < scalar(@ports_loc)) {
			$port = $ports_loc[$i];
			print "    [".($glob_i + 1)."]: Port ".$port."; PID: ".$loc_servers{$port}."; owner: ".$all_servers_usr{$port}."\n";
			$i++;
			$glob_i++;
		}
		$i = 0;
		while ($i < scalar(@ports_sys)) {
			$port = $ports_sys[$i];
			print "    [".($glob_i + 1)."]: Port ".$port."; PID: ".$sys_servers{$port}."; owner: ".$all_servers_usr{$port}." (system)\n";
			$i++;
			$glob_i++;
		}
		print "    Please, specify the number of the server you want to stop: ";
		$port = <STDIN>;
		$port =~ s/[\r\n]//g;
		if (($port < 1) or ($port > $num_loc + $num_sys)) {
			die "    Wrong number specified.\n";
		}
		if ($port > $num_loc) {
			$port = $ports_sys[$port - 1 - $num_loc];
			$system_run = 1;
		}
		else {
			$port = $ports_loc[$port - 1];
		}
	}
}
else {
	if ($all_servers{$port}) {
		confirm("    Are you sure you want to stop the server? ");
		$system_run = 1 if ($sys_servers{$port});
	}
	else {
		my $msg = "    No ATK server using the port '$port' was found.\n";
		if (system("ps -A | grep 'atk-server\\.pl' >/dev/null") == 0) {
			confirm("$msg    It seems that a previous version of ATK server is running. killall will be used to stop it.\n    Continue? ");
			system("$sudo_cmd 'killall atk-server.pl'");
			exit;
		}
		else {
			die $msg;
		}
	}
}

if (($all_servers_usr{$port} ne $whoami) and !$system_run) {
	if ($< == 0) {          # real root
		confirm("\n    The server was started by the user '".$all_servers_usr{$port}."'.\n    Do you want to stop it, anyway? ");
	}
	else {
		confirm("\n    The server was started by the user '".$all_servers_usr{$port}."' and you may not have permissions to stop it.\n    Do you want to try, anyway? ");
	}
}

my $cmd = "$atk_dir/utils/stop_tests.pl ".$all_servers{$port};

if (($system_run and ($< != 0)) or ($use_sudo)) {
	print "\n    The command '$sudo_cmd' will be used to gain root access.\n    If you want to change this, run $0 <port> <sudo-command>\n\n";
	$cmd = "$sudo_cmd '$cmd'";
}

system($cmd);
exit($?);
