# LSB Test Execution Framework
# General Templates and Functions Module (Templates.pm)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Common;
use strict;
require Exporter;

@Common::ISA = 'Exporter';
@Common::EXPORT = qw(
        $is_packager_rpm
    	$cert_sys_host $cert_sys_base
    	&guess_platform &guess_test_arch
    	&autoflush_on &is_process_running
    );

system('which dpkg >/dev/null 2>&1');
our $is_packager_rpm = ($? ? 1 : 0);

# Certification System
our $cert_sys_host = 'www.linux-foundation.org';
our $cert_sys_base = '/lsb-cert/';

# Helps to determine platform name.
sub guess_platform($) {
    if ($_[0] =~ m/^(amd64|x86_64|x86-64)$/i) {
		return 'x86-64';
	}
    elsif ($_[0] =~ m/^(ia32|i[3-6]86|x86)$/i) {
		return 'x86';
	}
    elsif ($_[0] =~ m/^(ia64)$/i) {
		return 'IA64';
	}
    elsif ($_[0] =~ m/^(ppc|ppc32)$/i) {
		return 'PPC32';
	}
    elsif ($_[0] =~ m/^(ppc64)$/i) {
		return 'PPC64';
	}
    elsif ($_[0] =~ m/^(s390)$/i) {
		return 'S390';
	}
    elsif ($_[0] =~ m/^(s390x)$/i) {
		return 'S390X';
	}
    else {
		return 'unknown';
	}
}

# Determines the ELF file architecture
sub guess_test_arch($) {
	my ($file) = @_;
	return 'unknown' if ((!-f $file) or (-s $file < 20));
	if (open(FILE, $file)) {
		my $elfhdr;
		read(FILE, $elfhdr, 20);
		close(FILE);
		my @e_machine_bytes;
		# Store 2 bytes for e_machine in little-endian order
		if (substr($elfhdr, 5, 1) eq "\x02") {
			@e_machine_bytes = (substr($elfhdr, 19, 1), substr($elfhdr, 18, 1));    # Big-Endian
		}
		else {
			@e_machine_bytes = (substr($elfhdr, 18, 1), substr($elfhdr, 19, 1));    # Little-Endian
		}
		my $e_machine = ord($e_machine_bytes[0]) + 16 * ord($e_machine_bytes[1]);
		my $e_width = substr($elfhdr, 4, 1);            # 1: 32-bit; 2: 64-bit
		if (($e_machine == 0x03) and ($e_width eq "\x01")) {
			return 'x86';
		}
		elsif (($e_machine == 0x3e) and ($e_width eq "\x02")) {
			return 'x86-64';
		}
		elsif (($e_machine == 0x32) and ($e_width eq "\x02")) {
			return 'IA64';
		}
		elsif (($e_machine == 0x14) and ($e_width eq "\x01")) {
			return 'PPC32';
		}
		elsif (($e_machine == 0x15) and ($e_width eq "\x02")) {
			return 'PPC64';
		}
		elsif (($e_machine == 0x16) and ($e_width eq "\x01")) {
			return 'S390';
		}
		elsif (($e_machine == 0x16) and ($e_width eq "\x02")) {
			return 'S390X';
		}
		else {
			return 'unknown';
		}
	}
	else {
		return 'unknown';
	}
}

sub autoflush_on() {
	my $old_handle = select(STDOUT);
	$| = 1;
	select(STDERR);
	$| = 1;
	select($old_handle);
}

sub is_process_running($) {
	my ($PID) = @_;
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return (($test_line ne '') and ($test_line !~ m/^Z/i));
}

1;
