# LSB Test Execution Framework
# General Templates and Functions Module (Templates.pm)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

sub BEGIN {
	push @INC, '../../utils';
}

package Templates;
use strict;
use Exporter;
use Common;

@Templates::ISA = 'Exporter';
@Templates::EXPORT = (
	qw(
        %SERVER_PARAM %_GET %_POST %_COOKIE %CONFIG 
        &print_header &print_footer &show_error_dlg
        &escape &unescape
    ),
    @Common::EXPORT
);

my ($global_header, $global_footer);

our %SERVER_PARAM = ();
$SERVER_PARAM{'HOST'} =          ($ENV{'ATKM_HOST'} or '');
$SERVER_PARAM{'PEER_IP'} =       ($ENV{'ATKM_PEER_IP'} or '');
$SERVER_PARAM{'PORT'} =          ($ENV{'ATKM_PORT'} or '');
$SERVER_PARAM{'DOCUMENT_ROOT'} = ($ENV{'ATKM_DOCUMENT_ROOT'} or '');
$SERVER_PARAM{'APP_DATA'} =      ($ENV{'ATKM_APP_DATA'} or '');


if (open(FILE, $SERVER_PARAM{'DOCUMENT_ROOT'}.'/header.tpl')) {
	$global_header = '';
	while (<FILE>) {
		$global_header .= $_;
	}
	close(FILE);
}
if (open(FILE, $SERVER_PARAM{'DOCUMENT_ROOT'}.'/footer.tpl')) {
	$global_footer = '';
	while (<FILE>) {
		$global_footer .= $_;
	}
	close(FILE);
}
if (!$global_header or !$global_footer) {
	$global_header = "<html><body>\n";
	$global_footer = '</body></html>';
}

sub escape($) {
	$_[0] =~ s/([^0-9a-zA-Z_ ])/sprintf('%%%02X', ord($1))/eg;
	$_[0] =~ s/ /\+/g;
}

sub unescape($) {
	$_[0] =~ s/\+/ /g;
	$_[0] =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;
}

my @args;

our %_GET = ();
@args = split(/&/, $ENV{'ATKM_GET_ARGS'});
foreach (@args) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name) {
		unescape($name);
		if (defined($val)) {
			unescape($val);
		}
		else {
			$val = '1';
		}
		$_GET{$name} = $val;
	}
}

our %_POST = ();
@args = split(/&/, $ENV{'ATKM_POST_ARGS'});
foreach (@args) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name) {
		unescape($name);
		if (defined($val)) {
			unescape($val);
		}
		else {
			$val = '1';
		}
		$_POST{$name} = $val;
	}
}

our %_COOKIE = ();
my @cookies_list = split(/;\s*/, $ENV{'ATKM_COOKIES'});
foreach (@cookies_list) {
	my ($name, $val) = split(/=/, $_, 2);
	if ($name and defined($val)) {
		unescape($name);
		unescape($val);
		$_COOKIE{$name} = $val;
	}
}

# Get "<DOCUMENT_ROOT>/../.." path
my $tmp = $SERVER_PARAM{'DOCUMENT_ROOT'};
$tmp =~ s!/[^/]+/[^/]+$!!;
our %CONFIG = (
	'TESTS_DIR' => ($ENV{'ATKM_SYSTEM_RUN'} ? '/opt/lsb/bin' : $SERVER_PARAM{'APP_DATA'}.'/bin'),
	'UTILS_DIR' => "$tmp/utils",
	'RESULTS_DIR' => $SERVER_PARAM{'APP_DATA'}.'/results'
);

my %styles = (
	'conf' => '\$\$\$CONFIGURE_STYLE\$\$\$',
	'results' => '\$\$\$RESULTS_STYLE\$\$\$',
	'report' => '\$\$\$RESULTS_STYLE\$\$\$',
	'analysis' => '\$\$\$RESULTS_STYLE\$\$\$',
	'upload' => '\$\$\$RESULTS_STYLE\$\$\$',
	'help' => '\$\$\$HELP_STYLE\$\$\$',
	'about' => '\$\$\$ABOUT_STYLE\$\$\$',
	'admin' => '\$\$\$ADMIN_STYLE\$\$\$'
);

sub print_header($$) {
	$global_header =~ s/\$\$\$PAGE_TITLE\$\$\$/$_[0]/;
	# Find and highlight the active menu link
	foreach my $name (keys %styles) {
		if ($_[1] eq $name) {
			$global_header =~ s/$styles{$name}/-active/;
			last;
		}
	}
	# Set all other menu links to non-highlighted style
	foreach (values %styles) {
		$global_header =~ s/$_//;
	}
	if ($_[1]) {
		$global_header =~ s/\$\$\$HELP_REF\$\$\$/#$_[1]/;
	}
	else {
		$global_header =~ s/\$\$\$HELP_REF\$\$\$//;
	}
	print $global_header;
}

sub print_footer {
	if ($_[0]) {
		$global_footer =~ s/<\/body>//;
		$global_footer =~ s/<\/html>//;
	}
	print $global_footer;
}

# Remove old session profiles
my $tm = time();
if (opendir(DIR, $SERVER_PARAM{'APP_DATA'}.'/profiles')) {
	my @files = grep /^~session\./, readdir(DIR);
	foreach (@files) {
		my $name = $SERVER_PARAM{'APP_DATA'}."/profiles/$_";
		# Session expiration time is 7 days
		if ($tm - (stat($name))[8] > 7*86400) {
			unlink($name);
		}
	}
	closedir(DIR);
}

sub show_error_dlg($) {
	my ($error_text) = @_;
	my $error_show = (($error_text eq '') ? ' display: none;' : '');
	return <<DATA;
<noscript><h1 align="center"><font color="red" size="+1">WARNING! Your browser does not support JavaScript.</font></h1></noscript>
<div id="error_msg_collapsed" style="border: dashed 1px darkred; background-color: red; float: right; cursor: pointer; display: none;" onclick="javascript:document.getElementById('error_msg_area').style.display='';document.getElementById('error_msg_collapsed').style.display='none';" title="Expand">
<table border="0" cellpadding="1" cellspacing="1">
  <tr>
    <td align="left" style="white-space: nowrap;"><font color="white"><b>&nbsp;There are some problems!</b></font></td>
    <td width="15"></td>
    <td width="11"><img src="images/expand.png" alt="O" width="20" height="20" /></td>
  </tr>
</table>
</div>
<div id="error_msg_area" style="border: none;$error_show">
<table border="0" cellpadding="1" cellspacing="1" width="70%" align="center">
  <tr>
    <th style="border: dashed 1px darkred; background: red; color: white; font-size: large;">
      <table border="0" cellpadding="0" cellspacing="0" align="center">
        <tr>
          <td width="100%">Errors!</td>
          <td><img src="images/collapse.png" alt="_" width="20" height="20" style="cursor: pointer;" title="Collapse" onclick="javascript:document.getElementById('error_msg_area').style.display='none';document.getElementById('error_msg_collapsed').style.display='';" /></td>
        </tr>
      </table>
    </th>
  </tr>
  <tr><td align="center" style="border: dashed 1px darkred; border-top: none;">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr><td height="10" colspan="3"></td></tr>
      <tr><td width="50%"></td><td align="left" id="error_msg_text" style="font-size: medium; white-space: nowrap;">$error_text</td><td width="50%"></td></tr>
      <tr><td height="10" colspan="3"></td></tr>
    </table>
  </td></tr>
</table>
</div>
DATA
}

1;
