#!/usr/bin/perl -w

# LSB Test Execution Framework
# AJAX Server Module (ajax_srv.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my $error_text = "";
my $data = "";

my $output_xml = <<DATA;
HTTP/1.0 200 OK
Content-type: text/xml

<?xml version="1.0" encoding="UTF-8"?>
DATA

sub send_reply() {
	if ($error_text) {
		$error_text = "<ul><li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$error_text</font></li></ul>";
		$error_text =~ s/&/&amp;/g;
		$error_text =~ s/</&lt;/g;
		$error_text =~ s/>/&gt;/g;
		print $output_xml."<root>\n<error>".$error_text."</error>\n</root>";
	}
	else {
		print $output_xml."<root>\n".$data."</root>";
	}
}

sub list_dir() {
	if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
		$data .= "<profiles>\n";
		foreach (sort(grep(!/^[~\.]/, readdir(DIR)))) {
			$data .= "<name>$_</name>\n";
		}
		$data .= "</profiles>\n";
		closedir(DIR);
	}
	else {
		$error_text = "Could not obtain directory list!<br />($!)";
	}
}

if (!$_GET{"action"}) {
	$error_text = "Incorrect call parameters!";
}
elsif ($_GET{"action"} eq "delete_profile") {               # Delete profile
	if ($_GET{"profile"}) {
		if (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"}) {
			$error_text = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		}
		else {
			unlink($SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"});
			list_dir();
		}
	}
	else {
		$error_text = "No profile name specified!";
	}
}
elsif ($_GET{"action"} eq "save_profile") {                 # Save profile
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
		if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
			$error_text = "Could not create profile directory:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles";
		}
	}
	if (!$error_text) {
		if (!$_POST{"edit_profile_name"}) {
			$error_text = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/<>\|\?\*]/) {
			$error_text = "Invalid profile name: ".$_POST{"edit_profile_name"};
		}
	}
	if (!$error_text) {
		if ($_POST{"application"} and ($_POST{"application"} =~ m/[\\\/<>\|\?\*]/)) {
			$error_text = "Invalid application name: ".$_POST{"application"};
		}
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_test_arch($CONFIG{'TESTS_DIR'}.'/lsbappchk'));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.2");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "core,c++,desktop");
			print FILE "APPLICATION:     ".$_POST{"application"}."\n" if ($_POST{"application"});
			if ($_POST{"paths"}) {
				$_POST{"paths"} =~ s/\r\n/\n/g;
				my @paths_lines = split(/\n/, $_POST{"paths"});
				foreach (@paths_lines) {
					print FILE "PATH:            $_\n" if ($_);
				}
			}
			if ($_POST{"packages"}) {
				$_POST{"packages"} =~ s/\r\n/\n/g;
				my @packages_lines = split(/\n/, $_POST{"packages"});
				foreach (@packages_lines) {
					print FILE "PACKAGE:         $_\n" if ($_);
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";
			print FILE "SHOW_HIDDEN:     ".$_POST{"show_hidden_files"}."\n" if (defined($_POST{"show_hidden_files"}));
			print FILE "LAST_DIR:        ".$_POST{"last_opened_dir"}."\n" if ($_POST{"last_opened_dir"});

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}
			close(FILE);
			list_dir();
		}
		else {
			$error_text = "Cannot open file for writing ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "load_profile") {                 # Load profile
	my %profile_data = (
		"application" => "",
		"paths" => "",
		"packages" => "",
		"architecture" => "",
		"lsb_version" => "3.2",
		"lsb_profile" => "core,c++,desktop",
		"show_hidden_files" => "1",
		"last_opened_dir" => "/",
		"comments" => ""
	);
	if (!$_GET{"profile"}) {
		$error_text = "No profile name specified!";
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		if (open(FILE, $full_save_name)) {
			my @comments = ();
			my @paths = ();
			my @packages = ();
			while (<FILE>) {
				s/[\r\n]//g;
				if (/^APPLICATION:\s+(.*)$/) {
					$profile_data{"application"} = $1;
				}
				elsif (/^PATH:\s+(.*)$/) {
					push @paths, $1;
				}
				elsif (/^PACKAGE:\s+(.*)$/) {
					push @packages, $1;
				}
				elsif (/^LSB_VERSION:\s+(.*)$/) {
					$profile_data{"lsb_version"} = $1;
				}
				elsif (/^LSB_PROFILE:\s+(.*)$/) {
					$profile_data{"lsb_profile"} = $1;
				}
				elsif (/^ARCHITECTURE:\s+(.*)$/) {
					$profile_data{"architecture"} = guess_platform($1);
				}
				elsif (/^SHOW_HIDDEN:\s+(.*)$/) {
					$profile_data{'show_hidden_files'} = $1;
				}
				elsif (/^LAST_DIR:\s+(.*)$/) {
					$profile_data{'last_opened_dir'} = $1;
				}
				elsif (/^COMMENTS: (.*)$/) {
					push @comments, $1;
				}
			}
			close(FILE);
			$profile_data{"paths"}    = join("\n", @paths);
			$profile_data{"packages"} = join("\n", @packages);
			$profile_data{"comments"} = join("\n", @comments);
			$data .= "<profile_data>\n";
			foreach (keys %profile_data) {
				$data .= "<item><name>$_</name><value>".$profile_data{$_}."</value></item>\n";
			}
			$data .= "</profile_data>\n";
			list_dir();
		}
		else {
			$error_text = "Cannot open file for reading ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "stop_tests") {                   # Stop the tests
	if (!$_GET{"tree"} or ($_GET{"tree"} ne "server")) {
		$error_text = "Incorrect call parameters!";
	}
	else {
		my $PID;
		`head -n 1 $SERVER_PARAM{APP_DATA}/log/atk-server.log.$SERVER_PARAM{PORT} 2>/dev/null` =~ m/PID:\s*(\d+)/i;
		$PID = $1;
		system($CONFIG{"UTILS_DIR"}."/stop_tests.pl $PID ".$SERVER_PARAM{"APP_DATA"}."/log/atk-server.log.".$SERVER_PARAM{"PORT"}." >/tmp/atk-stop-tests.out.".$SERVER_PARAM{"PORT"}." 2>&1");
		if ($? == 0) {
			$data = "<stopped id=\"".$_GET{"tree"}."\">1</stopped>\n";
		}
		else {
			$error_text = `cat /tmp/atk-stop-tests.out.$SERVER_PARAM{PORT}`;
		}
		unlink("/tmp/atk-stop-tests.out.".$SERVER_PARAM{"PORT"});
	}
}
else {
	$error_text = "Incorrect call parameters!";
}

send_reply();
