#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Page Module (tests_conf.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @lsb_versions_list = qw/3.2 3.1 3.0/;
my %lsb_profiles_list = ('core,c++' => 'Core &amp; C++', 'core,c++,desktop' => 'Core &amp; C++ &amp; Desktop');

my %archs_list = ();

my @error_msg = ();
my @warn_msg = ();

my %default_profile_data = (
	'application' => '',
	'paths' => '',
	'packages' => '',
	'architecture' => '',
	'lsb_version' => '3.2',
	'lsb_profile' => 'core,c++,desktop',
	'show_hidden_files' => '1',
	'last_opened_dir' => '/',
	'comments' => ''
);

my %profile_data = %default_profile_data;

# Determine architecture of the lsbappchk binary to use it as the only possible value (yet).
$default_profile_data{'architecture'} = guess_test_arch($CONFIG{'TESTS_DIR'}.'/lsbappchk');
if ($default_profile_data{'architecture'} eq 'unknown') {
	push @warn_msg, 'Could not detect the lsbappchk architecture.<br /><b>x86</b> is taken.';
	$default_profile_data{'architecture'} = 'x86';
}
$profile_data{'architecture'} = $default_profile_data{'architecture'};

my $error_text = '';
my $warn_text = '';
my $empty_output = 1;

if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
	system('mkdir -p '.$SERVER_PARAM{'APP_DATA'}.'/profiles');
	if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = '';
my $create_session_error = 0;

if (!$_COOKIE{'session_id'} or !-f $SERVER_PARAM{'APP_DATA'}.'/profiles/~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_COOKIE{'session_id'}) {
	my $i = time();
	while (-f $SERVER_PARAM{'APP_DATA'}.'/profiles/~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$i) {
		$i++;
	}
	$_COOKIE{'session_id'} = $i;
	$profile_name = '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$i;
	if (open(FILE, '>'.$SERVER_PARAM{'APP_DATA'}."/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_COOKIE{'session_id'};
}

if (open(FILE, $SERVER_PARAM{'APP_DATA'}."/profiles/$profile_name")) {
	my @comments = ();
	my @paths = ();
	my @packages = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^APPLICATION:\s+(.*)$/) {
			$profile_data{'application'} = $1;
		}
		elsif (/^PATH:\s+(.*)$/) {
			push @paths, $1;
		}
		elsif (/^PACKAGE:\s+(.*)$/) {
			push @packages, $1;
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{'lsb_version'} = $1;
		}
		elsif (/^LSB_PROFILE:\s+(.*)$/) {
			$profile_data{'lsb_profile'} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{'architecture'} = guess_platform($1);
		}
		elsif (/^SHOW_HIDDEN:\s+(.*)$/) {
			$profile_data{'show_hidden_files'} = $1;
		}
		elsif (/^LAST_DIR:\s+(.*)$/) {
			$profile_data{'last_opened_dir'} = $1;
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
	}
	close(FILE);
	$profile_data{'paths'}    = join("\n", @paths);
	$profile_data{'packages'} = join("\n", @packages);
	$profile_data{'comments'} = join("\n", @comments);
}
elsif (!$create_session_error) {
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if ($_COOKIE{'last_error_text'}) {
	push @error_msg, $_COOKIE{'last_error_text'};
}

# Set cookie expiration to 7 days since now
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time() + 7*86400);
my @wday_names = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
my @mon_names = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
# Format date like follows: Tuesday, 31-Dec-19 21:00:00 GMT
my $fmt_date = sprintf('%s, %d-%s-%d %02d:%02d:%02d GMT', $wday_names[$wday], $mday, $mon_names[$mon], 1900 + $year, $hour, $min, $sec);
print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: session_id=".$_COOKIE{'session_id'}."; expires=$fmt_date; path=/;\n";
print "Set-Cookie: last_error_text=; path=/;\n\n";

print_header('Application Check', 'conf');

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = '<ul>';
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= '</ul>';
}

my $table_spacer = '<td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>';

print show_error_dlg($error_text);

print <<DATA;

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute;">
<table cellpadding="0" cellspacing="0" border="0" style="width: 17em;">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:hide_tooltip();" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<div id="ajax_loading" style="display: none; position: fixed; width: 13em; height: 5.8em; top: 40%; left: 50%; margin-top: -2.9em; margin-left: -7.5em; border: solid 1px #00007f; background-color: #e7f0ff; text-align: center; padding: 6px;"><div align="center">
Please, wait&hellip;<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" style="border: solid 1px #00007f; background: #e0e0ff; color: blue; width: 6em;" onclick="javascript:onAbort();" />
</div></div>

<h1>Application Check</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test" id="configure_test">
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<input type="hidden" name="show_hidden_files" id="show_hidden_files" value="$default_profile_data{show_hidden_files}" />
<input type="hidden" name="last_opened_dir" id="last_opened_dir" value="$default_profile_data{last_opened_dir}" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td valign="top" style="white-space: nowrap;">Application&nbsp;name:&nbsp;<font color="red">*</font></td>
          $table_spacer
          <td width="100%" colspan="2"><input type="text" name="application" id="application" value="$default_profile_data{application}" style="width: 100%;" onfocus="javascript:style.borderColor=input_border_color;" /></td>
        </tr>
        <tr>
          <td valign="top" style="white-space: nowrap;">Files&nbsp;and&nbsp;directories:<br /><span style="font-size: 125%;">[</span><a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip('file_formats', event); return false;">Supported&nbsp;formats&nbsp;<img src="images/question.png" width="16" height="17" alt="?" /></a><span style="font-size: 125%;">]</span></td>
          $table_spacer
          <td width="100%" colspan="2"><textarea name="paths" id="paths" rows="5" cols="58" style="width: 100%;">$default_profile_data{paths}</textarea></td>
        </tr>
        <tr>
          <td width="100%" colspan="4" align="right"><input type="button" name="browse_for_file_but" value="Select Files/Dirs&hellip;" style="width: 10em;" onclick="javascript:browse_for_file_dir();" /></td>
        </tr>
        <tr>
          <td valign="top" style="white-space: nowrap;">Installed packages:</td>
          $table_spacer
          <td width="100%" colspan="2"><textarea name="packages" id="packages" rows="5" cols="58" style="width: 100%;">$default_profile_data{packages}</textarea></td>
        </tr>
        <tr>
          <td width="100%" colspan="4" align="right"><input type="button" name="browse_for_pkg_but" value="Select Packages&hellip;" style="width: 10em;" onclick="javascript:browse_for_pkg();" /></td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td valign="middle" colspan="4">
            <table border="0" cellpadding="0" cellspacing="0" width="100%">
              <tr>
                <td>Architecture:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td>
                  <select name="architecture" id="architecture" style="width: 6em;" disabled="disabled">
DATA

my @archs;

if ($default_profile_data{'architecture'} =~ m/unknown/i) {
	@archs = qw/x86 x86-64 IA64 PPC32 PPC64 S390 S390X/;
}
elsif ($default_profile_data{'architecture'} =~ m/x86-64/i) {
	@archs = qw/x86 x86-64/;
}
elsif ($default_profile_data{'architecture'} =~ m/IA64/i) {
	@archs = qw/x86 IA64/;
}
elsif ($default_profile_data{'architecture'} =~ m/PPC64/i) {
	@archs = qw/PPC32 PPC64/;
}
elsif ($default_profile_data{'architecture'} =~ m/S390X/i) {
	@archs = qw/S390 S390X/;
}
else {
	@archs = ($default_profile_data{'architecture'});
}

foreach (sort @archs) {
	if ($_ eq $default_profile_data{'architecture'}) {
		print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                    <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td width="100%" align="center">
                  <table cellpadding="0" cellspacing="0" border="0">
                    <tr>
                      <td>LSB&nbsp;Version:</td>
                      <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                      <td>
                        <select name="lsb_version" id="lsb_version" style="width: 6em;" onchange="javascript:control_profile_list();">
DATA

foreach (@lsb_versions_list) {
	if ($_ eq $default_profile_data{'lsb_version'}) {
		print "                          <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                          <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
                        </select>
                      </td>
                    </tr>
                  </table>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>LSB&nbsp;Profile:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td width="100%">
                  <select name="lsb_profile" id="lsb_profile" style="width: 15em;">
DATA

foreach (sort keys %lsb_profiles_list) {
	if ($_ eq $default_profile_data{'lsb_profile'}) {
		print "                    <option value=\"$_\" selected=\"selected\">$lsb_profiles_list{$_}</option>\n";
	}
	else {
		print "                    <option value=\"$_\">$lsb_profiles_list{$_}</option>\n";
	}
}

print <<DATA;
                  </select>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4"><img src="images/environment-spacer.gif" width="1" height="5" border="0" alt="" /></td>
        </tr>
        <tr>
          <td valign="top">Comments:</td>
          $table_spacer
          <td colspan="2"><textarea name="comments" id="comments" rows="5" cols="64" style="width: 100%;">$default_profile_data{comments}</textarea></td>
        </tr>
        <tr>
          <td colspan="4"><img src="images/environment-spacer.gif" width="1" height="3" border="0" alt="" /></td>
        </tr>
        <tr>
          <td colspan="4" align="right"><input type="reset" name="reset_button" id="reset_button" value="Clear Form" style="width: 11em;" onclick="javascript:reset_form(); return false;" />&nbsp;&nbsp;&nbsp;<input type="submit" name="run_tests" id="run_tests_button" value="Run the Test" style="width: 11em; font-weight: bold;" onclick="javascript:return do_action('run_tests');" /></td>
        </tr>
      </table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = '';
my @profiles_list;
if (opendir(DIR, $SERVER_PARAM{'APP_DATA'}.'/profiles')) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "              <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = '';
if ($profiles_options eq '') {
	$profiles_options = "              <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = ' disabled="disabled"';
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

my $array_profiles = ((scalar(@profiles_list) > 0) ? '\''.join('\', \'', @profiles_list).'\'' : '');

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip('profile_management', event); return false;"><img src="images/question.png" width="16" height="17" alt="?" /></a></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name:</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 18em;" onkeypress="javascript:return onKeyPressEdit(event);" onfocus="javascript:style.borderColor=input_border_color;" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="width: 18.1em;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event);">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="button" name="save_profile_button" id="save_profile_button" value="Save" style="width: 5.8em;" onclick="javascript:onSave();" /></td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 5.8em;" onclick="javascript:onLoad();" /></td>
          <td width="100%"><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 5.8em;" onclick="javascript:onDelete();" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
// Preload some images that are not initially visible
var preload_image_names = new Array('expand.png', 'collapse.png', 'drop_down_pressed.png', 'close.png');
var preload_images = new Array();
for (var i=0; i<preload_image_names.length; ++i) {
	preload_images[i] = new Image;
	preload_images[i].src = 'images/' + preload_image_names[i];
}

// Cache the necessary elements of the page
var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var app_name = document.getElementById('application');
if (!list_profile_name || !edit_profile_name || !hidden_div || !drop_down_button || !app_name)
	alert('Internal error: Cannot find essential form fields! JavaScript will not work correctly.\\nPlease, contact the authors.');

var input_border_color = ((navigator.appName == 'Konqueror') ? '#cccccc' : '');
var profiles_list = new Array($array_profiles);

// Writes down all the profile data into the form
function apply_profile_data() {
	var profile_data = new Array();
DATA

foreach my $field (keys %profile_data) {
	my $data = $profile_data{$field};
	$data =~ s/\r?\n/\\n/g;
	print "\tprofile_data['$field'] = '$data';\n";
}

print <<DATA;
	for (var field in profile_data) {
		var elem = document.getElementById(field);
		if (elem)
			elem.value = profile_data[field];
	}
}

// Processing any significant action
function do_action(name) {
	var elem = document.getElementById('form_action');
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease, contact the authors.");
		return false;
	}
	if (name == 'run_tests') {          // Maybe, remove parameter 'name' at all...
		if (!app_name || (app_name.value == '')) {
			app_name.style.borderColor = 'red';
			alert('Please, enter the application name!');
			return false;
		}
		else if (app_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
			app_name.style.borderColor = 'red';
			alert('The application name should contain only symbols that are allowed in file names!');
			return false;
		}
		if ((document.getElementById('paths').value == '') && (document.getElementById('packages').value == '')) {
			alert('You should specify something to test!');
			return false;
		}
	}
	elem.value = name;
	return true;
}

function reset_form() {
	if (confirm('Are you sure you want to clear all fields?')) {
		document.configure_test.reset();
		control_profile_list();
	}
}

// Tooltip processing functions
var last_tooltip = '';
var tooltip_texts = new Array();
tooltip_texts['profile_management'] = 'This section allows you to save all the settings<br />from the left pane into user profiles and load them afterwards.';
tooltip_texts['file_formats'] = 'Supported file formats:<ul style="line-height: 1.2em; margin-top: 0;"><li>ELF (executable and shared object);</li><li>Perl scripts;</li><li>Python scripts;</li><li>Shell scripts.</li></ul>You can specify also:<ul style="line-height: 1.2em; margin-top: 0;"><li>directories;</li><li>RPM packages;</li><li>TAR.GZ packages;</li></ul>to test them with full contents.';

var just_shown_tooltip = false;

function show_hide_tooltip(id, event) {
	var tt_window = document.getElementById('tooltip');
	if (tt_window) {
		if (last_tooltip != id) {
			var tt_text = document.getElementById('tooltip_text');
			if (tt_text) {
				if (tooltip_texts[id])
					tt_text.innerHTML = tooltip_texts[id] + '<br /><div align="right"><a href="tests_help.pl#' + id + '" target="_blank" onclick="javascript:hide_tooltip();"><i>Details&hellip;</i></a></div>';
				else
					tt_text.innerHTML = '<font color="red"><b>Sorry, no information for this item was found!</b></font><br />Please, write to <a href="mailto:linux\@ispras.ru">linux\@ispras.ru</a>.';
				var ScrollTop = document.body.scrollTop;
				if (ScrollTop == 0)
				{
				    if (window.pageYOffset)
				        ScrollTop = window.pageYOffset;
				    else
				        ScrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
				}
				var ScrollLeft = document.body.scrollLeft;
				if (ScrollLeft == 0)
				{
				    if (window.pageXOffset)
				        ScrollLeft = window.pageXOffset;
				    else
				        ScrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
				}
				var div_top = event.clientY + ScrollTop + 10;
				var div_left = event.clientX + ScrollLeft + 10;
				tt_window.style.display = '';
				var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
				var wnd_width = (document.body.clientWidth || window.innerWidth);
				if (div_left + div_width > wnd_width)
					div_left = wnd_width - div_width;
				tt_window.style.top = div_top + 'px';
				tt_window.style.left = div_left + 'px';
				last_tooltip = id;
				just_shown_tooltip = true;
			}
		}
		else {
			tt_window.style.display = 'none';
			last_tooltip = '';
		}
	}
}

function hide_tooltip() {
	var elem = document.getElementById('tooltip');
	if (elem) {
		elem.style.display = 'none';
		last_tooltip = '';
	}
}

// Editable drop-down list functions
var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute('src', 'images/drop_down_pressed.png');
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute('src', 'images/drop_down.png');
}

function onBodyClick() {
	if (hidden_div.style.display != 'none') {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_tooltip)
		hide_tooltip();
	just_shown_tooltip = false;
}

function onKeyPressList(event) {
	if (hidden_div.style.display == 'none')
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (hidden_div.style.display == 'none')
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

// Auto-saving user profile functions
function save_and_go(event) {
	var url = this.getAttribute('href');
	// IE fix
	if (!event)
		event = window.event;
	document.configure_test.target = (event.shiftKey ? '_blank' : '');
	document.configure_test.action = 'tests_conf_save.pl?goto=' + url.replace('#', '%23');
	document.configure_test.submit();
	document.configure_test.action = 'tests_conf_save.pl';
	document.configure_test.target = '';
	event.cancelBubble = true;
	return false;
}

function modify_all_links() {
	var links = document.getElementsByTagName('A');
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == '') && (links[i].href != '') && (links[i].onclick == null)) {
			links[i].onclick = save_and_go;
		}
	}
}

// LSB Profile management functions
function control_profile_list() {
	var elem_v = document.getElementById('lsb_version');
	var elem_p = document.getElementById('lsb_profile');
	if (!elem_v || !elem_p)
		return;
	if (elem_v.value != '3.0') {
		elem_p.disabled = false;
	}
	else {
		elem_p.disabled = true;
		elem_p.value = 'core,c++,desktop';
	}
}

// Browse functions
function browse_for_pkg() {
	window.open('choose_pkg.pl', 'browse_for_pkg', 'scrollbars=yes,resizable=yes,width=550,height=450').focus();
}

function browse_for_file_dir() {
	var elem;
	elem = document.getElementById('show_hidden_files');
	var show_hidden_files = (elem ? elem.value : '1');
	elem = document.getElementById('last_opened_dir');
	var last_opened_dir = (elem ? elem.value : '/');
	window.open('choose_file.pl?root=' + encodeURIComponent(last_opened_dir) + '&hidden=' + show_hidden_files, 'browse_for_file_dir', 'scrollbars=yes,resizable=yes,width=550,height=450').focus();
}

// AJAX
var req;
var busy = false;
var ajax_srv_url = document.location.protocol + '//' + document.location.host + '/ajax_srv.pl';
var progressDiv = document.getElementById('ajax_loading');
if (navigator.appName == 'Microsoft Internet Explorer')
	progressDiv.style.height = '7.5em';

var err_area = document.getElementById('error_msg_area');
var err_text = document.getElementById('error_msg_text');

function setBusyOn() {
	busy = true;
	setTimeout('showAjaxProgress()', 250);
}

function showAjaxProgress() {
	if (busy)
		progressDiv.style.display = '';
}

function setBusyOff() {
	busy = false;
	progressDiv.style.display = 'none';
}

function onSave() {
	if (edit_profile_name.value == '') {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
	else if (edit_profile_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
		edit_profile_name.style.borderColor = 'red';
		alert('The profile name should contain only symbols that are allowed in file names!');
	}
	else if (app_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
		app_name.style.borderColor = 'red';
		alert('The application name should contain only symbols that are allowed in file names!');
	}
	else {
		for (var i=0; i<profiles_list.length; ++i) {
			if (edit_profile_name.value == profiles_list[i]) {
				if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
					return;
				break;
			}
		}
		ajax_call_post('save_profile');
	}
}

function onLoad() {
	if (edit_profile_name.value) {
		ajax_call_get('load_profile&profile=' + edit_profile_name.value);
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function onDelete() {
	if (edit_profile_name.value) {
		if (confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
			ajax_call_get('delete_profile&profile=' + edit_profile_name.value);
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please, specify the profile name!');
	}
}

function onAbort() {
	setBusyOff();
	req.abort();
}

function ajax_call_get(action) {
	if (!busy) {
		setBusyOn();
		loadXMLDoc(ajax_srv_url + '?action=' + action, 'GET', null);
	}
}

function ajax_call_post(action) {
	if (!busy) {
		setBusyOn();
		loadXMLDoc(ajax_srv_url + '?action=' + action, 'POST', getFormValues(document.getElementById('configure_test')));
	}
}

function getFormValues(fobj)
{
	var str = '';
	for(var i=0; i<fobj.elements.length; ++i)
	{
		if ((fobj.elements[i].type == 'hidden') || (fobj.elements[i].type == 'text') ||
			(fobj.elements[i].type == 'textarea') || (fobj.elements[i].type == 'select-one'))
		{
			if (fobj.elements[i].value)
				str += fobj.elements[i].name + '=' + encodeURIComponent(fobj.elements[i].value) + '&';
		}
	}
	str = str.substr(0, (str.length - 1));
	return str;
}

function loadXMLDoc(url, method, post_params) {
	if (window.XMLHttpRequest) {        // native XMLHttpRequest object
		req = new XMLHttpRequest();
	}
	else if (window.ActiveXObject) {    // IE/Windows ActiveX version
		try {
			req = new ActiveXObject('Msxml2.XMLHTTP');
		}
		catch (e) {
			try {
				req = new ActiveXObject('Microsoft.XMLHTTP');
			}
			catch (e) {
				alert('Sorry, your browser does not support AJAX');
				return false;
			}
		}
	}
	if (!req) {
		alert('Sorry, your browser does not support AJAX');
		return false;
	}

    req.onreadystatechange = processReqChange;
    req.open(method, url, true);
	req.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	req.setRequestHeader('If-Modified-Since', 'Tue, 01 Jan 1980 00:00:00 GMT');
	req.setRequestHeader('Content-length', (post_params ? post_params.length : 0));
	req.setRequestHeader('Connection', 'close');
    req.send(post_params);
}

function processReqChange() {
	if (req.readyState == 4) {     // Finished
		setBusyOff();
        if (req.status == 200) {
			if (req.responseXML.getElementsByTagName('error').length > 0) {
	            var error_text = req.responseXML.getElementsByTagName('error')[0].childNodes[0].nodeValue;
            	err_text.innerHTML = error_text;
            	err_area.style.display = '';
			}
			else {
            	err_area.style.display = 'none';
				if (req.responseXML.getElementsByTagName('profiles').length > 0) {
		            var elem_list = document.getElementById('list_profile_name');
		            var elem_div = elem_list.parentNode;
		            elem_div.removeChild(elem_list);
		            elem_list = document.createElement('select');
		            elem_list.id = 'list_profile_name';
		            elem_list.name = 'list_profile_name';
		            elem_list.style.width = '18.1em';
		            elem_list.onclick = onClickList;
		            elem_list.onkeypress = onKeyPressList;
		            elem_div.appendChild(elem_list);
		            list_profile_name = elem_list;
		            profiles_list = new Array();

		            var profile_names = req.responseXML.getElementsByTagName('profiles')[0].getElementsByTagName('name');
		            if (profile_names.length > 0) {
			            for (var i=0; i<profile_names.length; ++i) {
			            	profiles_list[i] = profile_names[i].childNodes[0].nodeValue;
			            	var tmp_node = document.createElement('option');
			            	tmp_node.setAttribute('value', profiles_list[i]);
			            	tmp_node.appendChild(document.createTextNode(profiles_list[i]));
			            	elem_list.appendChild(tmp_node);
			            }
		            	elem_list.disabled = false;
			            if (profile_names.length < 2)
			            	elem_list.size = 2;
			            else if (profile_names.length > 6)
			            	elem_list.size = 6;
			            else
			            	elem_list.size = profile_names.length;
			        }
			        else {
		            	var tmp_node = document.createElement('option');
		            	tmp_node.setAttribute('value', '');
		            	tmp_node.appendChild(document.createTextNode('<No profiles present>'));
		            	elem_list.appendChild(tmp_node);
		            	elem_list.disabled = true;
		            	elem_list.size = 2;
			        }
	            }
				if (req.responseXML.getElementsByTagName('profile_data').length > 0) {
					document.configure_test.reset();
		            var profile_data = req.responseXML.getElementsByTagName('profile_data')[0].getElementsByTagName('item');
		            for (var i=0; i<profile_data.length; ++i) {
						var item_name = profile_data[i].getElementsByTagName('name')[0].childNodes[0].nodeValue;
						var item_values = profile_data[i].getElementsByTagName('value')[0].childNodes;
						var item_value = '';
						if (item_values.length > 0)
							item_value = item_values[0].nodeValue;
		            	var form_elem = document.getElementById(item_name);
		            	if (form_elem)
		            		form_elem.value = item_value;
		            }
		            control_profile_list();
				}
				edit_profile_name.value = '';
			}
        }
        else {
            alert('Error in AJAX call:\\n' + req.statusText);
        }
    }
}

// Startup actions
apply_profile_data();
control_profile_list();
modify_all_links();
// ]]>
</script>
DATA

print_footer();
